program DiskIdent;

{-----------------------------------------------------------------------

                           D S K I D . P A S
  schreibt einen Identifikator auf einen von CP/M nicht erreichbaren
  Disketten-Sektor ( nur fuer Typ 03- und Typ 07-Laufwerke)

                             Wolfgang Tesch
                                30-12-85
---------------------------------------------------------------------------}


type
  Puffer = array[0..127] of byte;

var
  Spur,
  Sektor,
  Fehler,
  LwNr    : byte;
  DMA     : Puffer ;
  Ident   : string[10];
  i       : byte;



procedure Error (Nr : byte);
  const
    Msg1 = 'ungueltige Laufwerksbezeichnung';
    Msg2 = 'bei diesem Laufwerkstyp ist kein Schreiben eines Identifikators ';
    Msg2a= 'moeglich';
    Msg10= 'Identifikator geschrieben';
    Msg3 = 'kein Identifikator geschreiben';

  begin
    writeln;
    case Nr of
     1  :  writeln(Msg1);
     2  :  begin
             write(Msg2);writeln(Msg2a);
           end;
     3  :  writeln(Msg3);
     10 :  writeln(Msg10);
    end;
    BDOS(0); { Programmabbruch,Kontrolle an CCP }
  end;



procedure SelLaufw ( var LwNr : byte );

{ in welchem Laufwerk ist die Diskette, auf die geschrieben werden soll ? }

  begin
    if ParamCount = 0
      then { nehme aktuelles Laufwerk }
        LwNr := mem[$0004]
      else { untersuche 'command line parameter' }
        if ParamStr(1) in ['A'..'I']
          then
            LwNr := ord(ParamStr(1)) - ord('A')
          else
            Error(1);

    { Laufwerk anmelden :}
    BDOS(14,LwNr);

    { wurde ein nicht verfuegbares Laufwerk angegeben, so wird das Programm
      an dieser Stelle mit der Meldung 'Bdos Err on ..: Select' abgebrochen }
  end;



procedure TestConfig ( var Spur,Sektor : byte);

{ Teste, ob gewaehltes Laufwerk vom Typ 03 oder 07 ist.
  Falls ja, lege Spur und Sektor fest, an denen der Identifikator geschreiben
  werden soll }

const
  { Spur/Sektor fuer Typ 03 }
  T3Spur   = 98;
  T3Sektor =  1;
  { Spur/Sektor fuer Typ 07 }
  T7Spur   = 195;
  T7Sektor =  13;

  CFGBYT = $FFE9; { enthaelt den CONFIG-Typ }

  procedure WriteHexByte ( b :byte);
  { Prozedur zum Ausdrucken eines Bytes als Hex-Zahl }
  begin
    Inline($CD/$04B4);
  end;

begin
  case mem[CFGBYT] of
   03  : begin
           Spur   := T3Spur;
           Sektor := T3Sektor;
         end;
   07  : begin
           Spur   := T7Spur;
           Sektor := T7Sektor;
         end;
  else { kein Schreiben eines Identifikators moeglich }
    begin
      writeln;
      write('Laufwerk vom Typ ');
      WriteHexByte( mem[CFGBYT] );
      Error(2);
    end;
  end;
end;



procedure LiesSektor( Spur,Sektor : byte;
                      var Fehler : byte );

{ die Prozedur liest den Inhalt des durch  Spur und Sektor spezifizierten
  logischen Sektor in einen Puffer, der im aufrufenden Programm spezifiziert
  sein muss }

begin
  { Spur/Sektor auswaehlen }
  BIOS( 9,Spur);
  BIOS(10,Sektor);
  { Lesen }
  Fehler := BIOS(12);
end;




procedure SchreibSektor(Spur,Sektor:byte;var Fehler:byte);

{ die Prozedur schreibt den Inhalt des Puffers in den spezifizierten
  log. Sektor }

begin
  { Spur/Sektor auswaehlen }
  BIOS( 9,Spur);
  BIOS(10,Sektor);
  { Schreiben }
  Fehler := BIOS(13);
end;



BEGIN
  writeln; writeln;
  writeln('D I S K I D E N T  :');
  writeln;
  writeln('Programm zum Schreiben eines Disketten-Identifikators ');
  writeln('fuer Laufwerke vom Typ 03 oder 07');
  writeln; writeln;

  SelLaufw( LwNr );
  TestConfig ( Spur, Sektor);
  writeln('Identifikator wird auf Diskette in Laufwerk ',chr(LwNr+ord('A')),
          ': geschrieben');

  { DMA-Puffer festlegen }
  BDOS(26,addr(DMA));

  LiesSektor( Spur, Sektor, Fehler );
  { stelle fest, ob Sektor schon beschrieben }
  if DMA[1] <> $E5  { $E5 wird beim Formatieren geschreiben und zeigt
                      leeren Sektor an }
    then
    begin
      writeln;
      write ('Sektor enthaelt Information : ');
      for i := 0 to 9 do
        if DMA[i] in [32..127]
          then write(chr(DMA[i]))
          else write(' ');
      writeln;
      write('ueberschreiben ? (J/N) ');
      readln(Ident);
      if upcase(Ident[1]) <> 'J' then Error(3);
    end;

  writeln;
  write('Identifikator ? ');
  readln(Ident);
  for i := 0 to length(Ident)-1 do
    DMA[i] := ord(Ident[i+1]);
  for i := length(Ident) to 127 do
    DMA[i] := ord(' ');
  SchreibSektor(Spur,Sektor,Fehler);
  Error(10);

END.