{****************************************************************************}
{*                           Maskeneingabe                                  *}
{****************************************************************************}
{Dieses Modul beinhaltet zwei Prozeduren zur komfortablen Eingabe von Records,
 deren Komponenten durchwegs STRINGS sind.

 Die Maske ist ein ARRAY [1..RecEintrZahl] of MaskEintrag,
 ein MaskEintrag ein RECORD x,y : BYTE;       (=Bildschirmposition)
                            m   : STRING[80]; (=Meldung)
                            l   : BYTE;       (=max.Laenge des Eintrags)
                     END;
 x,y und m sind beliebig, l muss der RECORD-Definition entsprechen.
 Beispiel:
   VAR USER_RECORD : RECORD
                       Eintrag1 : STRING[20];
                       Eintrag2 : STRING[30];
                       Eintrag3 : STRING[40]
                     END;
   CONST ANZAHL_DER_RECORDEINTRAEGE = 3; ( Anzahl der Eintraege )
         USER_MASKE : ARRAY [1..RecEintrZahl] of
                         RECORD x,y : BYTE; m : STRING[80]; l : BYTE end
                    = ((x: 1; y: 4; m:'Meldung eins'; l: 20),
                       (x:40; y: 4; m:'Meldung zwei'; l: 30),
                       (x:20; y: 5; m:'Meldung drei'; l: 40));
 Nach dieser Definition bewirkt der Aufruf
     SchreibRecord(USER_RECORD,USER_MASKE,ANZAHL_DER_RECORDEINTRAEGE)
 die masken-formatierte Ausgabe und
     LiesRecord(USER_RECORD,USER_MASKE,ANZAHL_DER_RECORDEINTRAEGE)
 die entsprechende Eingabe der VARiablen USER_RECORD.
 Die Editiermoeglichkeiten sind WORDSTAR-kompatibel. Die Eingabe wird erst
 dann abgeschlossen, wenn das letzte Feld mit der RETURN-Taste verlassen wird}

{****************************************************************************}
{*               Schreibt einen beliebigen Record mit Maske                 *}
{****************************************************************************}

  procedure SchreibRecord(var UsRec, UsMask; RecEintrZahl : byte);

    const IVON    = ^D'3'; {Inverses Bild an}
          IVOFF   = ^D'0'; {Inverses Bild aus}

    type anystring= string[255];

    var  temp     : anystring;
         i        : byte;
         Maske    : array [1..255] of record
                                      xp, yp : byte;
                                      m      : string[80];
                                      l      : byte
                                    end
                    absolute UsMask;
         RecFeld  : array [0..maxint] of byte absolute UsRec;

    procedure CopyRecTemp(x:byte);
    {kopiert den jeweils gueltigen Record-Eintrag in die Variable temp}
      var i,s : integer;
      begin
        s:=0; for i:=1 to x-1 do s:=s+Maske[i].l+1; temp:='';
        for i:=1 to RecFeld[s] do temp:=temp+chr(RecFeld[s+i])
      end;

    begin {SchreibRecord}
      for i:=1 to RecEintrZahl do
        with Maske[i] do
          begin
            gotoxy(xp,yp); write(IVON,M,IVOFF);
            CopyRecTemp(i);
            while length(temp)<l do temp:=temp+'_';
            write(' ',temp)
           end
    end;
{.PA}
{****************************************************************************}
{*                     LiesRecord mit Maske                                 *}
{****************************************************************************}

   procedure LiesRecord(var UsRec, UsMask; RecEintrZahl : byte);
   {$V-} {keine Range-Check bei String-Uebergabe}

     type Richtung = (keine,fertig,rauf,runter);
          anystring= string[255];

     var  selektor : byte;
          weiter   : Richtung;
          temp     : anystring;
          Maske    : array [1..255] of record
                                      xp, yp : byte;
                                      m      : string[80];
                                      l      : byte
                                    end
                    absolute UsMask;
          RecFeld : array [0..maxint] of byte absolute UsRec;

    procedure CopyRecTemp(x:byte);
    {kopiert den jeweils gueltigen Record-Eintrag in die Variable temp}
      var i,s : integer;
      begin
        s:=0; for i:=1 to x-1 do s:=s+Maske[i].l+1; temp:='';
        for i:=1 to RecFeld[s] do temp:=temp+chr(RecFeld[s+i])
      end;

    procedure CopyTempRec(x:byte);
    {kopiert die Variable temp in den jeweils gueltigen Record-Eintrag}
      var i,s : integer;
      begin
        s:=0; for i:=1 to x-1 do s:=s+Maske[i].l+1;
        RecFeld[s]:=length(temp);
        for i:=1 to length(temp) do RecFeld[s+i]:=ord(temp[i])
      end;

    procedure Eingabe(Spalte, Zeile : integer; var Antwort : anystring;
                      MaxLaenge : integer; var weiter : Richtung);
      var   CPos,
            i      : integer;
            c      : char;

      procedure beep;
        begin write(^G) end;

      begin
        gotoxy(Spalte,Zeile);
        CPos:=length(Antwort)+1;
        repeat
          gotoxy(Spalte+CPos-1,Zeile);
          read(kbd,c);
          case c of
            ^M  : weiter := fertig;
            ^E  : weiter := rauf;
            ^X  : weiter := runter;
            ^H  : if CPos>1
                    then begin
                           CPos:=pred(CPos); gotoxy(Spalte+CPos-1,Zeile);
                           delete(Antwort,CPos,1);
                           write(copy(Antwort,CPos,length(Antwort)-CPos+1),'_');
                         end
                    else beep;
            ^G  : if CPos>0
                    then begin
                           delete(Antwort,CPos,1);
                           write(copy(Antwort,CPos,length(Antwort)-CPos+1),'_');
                         end
                    else beep;
            ^S  : if CPos>1 then CPos:=pred(CPos) else beep;
            ^D  : if CPos<=length(Antwort) then CPos:=succ(CPos) else beep;
            ^Y  : begin
                    gotoxy(Spalte,Zeile); CPos:=0;
                    while CPos<length(Antwort) do
                      begin write('_'); CPos:=succ(CPos) end;
                   Antwort:=''; CPos:=1
                 end;
     #32..#126  : if length(Antwort)<MaxLaenge
                    then begin
                          insert(c,Antwort,CPos);
                          write(copy(Antwort,CPos,length(Antwort)-CPos+1));
                          CPos:=succ(CPos)
                         end
                   else beep;
          end
        until weiter<>keine
      end;

    begin {LiesRecord}
      selektor:=1;
      SchreibRecord(UsRec,Maske,RecEintrZahl);
        repeat
          weiter:=keine;
          CopyRecTemp(selektor);
          with Maske[selektor] do Eingabe(xp+length(M)+1,yp,temp,l,weiter);
          CopyTempRec(selektor);
          if weiter=rauf then if selektor>1 then selektor:=pred(Selektor)
                                            else selektor:=RecEintrZahl;
          if weiter=runter then if selektor<RecEintrZahl then selektor:=succ(selektor)
                                                       else selektor:=1;
          if weiter=fertig then if selektor<>RecEintrZahl
                              then begin selektor:=succ(selektor); weiter:=keine end;
        until (selektor=RecEintrZahl) and (weiter=fertig);
    end;
