program sierpinski;

{------------------------------------------------------------------------

                           S I E R P . P A S
                          -------------------
  Das Programm erstellt auf dem DMX80-Drucker ein Bild der Sierpinksi-
  Kurven der Ordnung 1..n.
  Dazu verwendet es Prozeduren aus dem Beitrag 'Grafik ohne Bildschirm'
  aus dem Chip-Sonderheft 'PC-Soft Anwenderprogramme in Turbo-Pascal'

                             Wolfgang Tesch
-------------------------------------------------------------------------}

const xmax = 512;
      ymax = 512;

type VBR = array[0..xmax] of byte;
     Zeile = string[80];
var GBS : array[0..64] of VBR;

procedure ClrGBS;
  begin
    fillchar(GBS,sizeof(GBS),#0);
  end;

procedure plot(x,y:integer);
  begin
    writeln('      y = ',y);
    y:=512-y;
    if (0<=x) and (x <= xmax) and (0 <= y) and (y <= ymax)
      then GBS[y SHR 3,x]:=GBS[y SHR 3,x] or (128 SHR (y mod 8 ));
  end;

procedure line(ax,ay,bx,by:integer);
  var g,i  : integer;
      dx,dy,l,s,c : real;

  begin
    dx := bx-ax;
    dy := by-ay;
    l := sqrt(sqr(dx)+sqr(dy));
    if l =0
      then plot (ax,ay)
      else
        begin
          s := dy/l;
          c := dx/L;
          g := round(l);
          for i:= 0 to g do
            plot(round(i*c)+ax,round(i*s)+ay);
        end;
  end;

procedure quad (x,y,z,v:integer);
  begin
    line (x,y,z,y);
    line (z,y,z,v);
    line (z,v,x,v);
    line (x,v,x,y);
  end;


var turtle : record
             xPosition : integer;
             yPosition : integer;
             Richtung  : integer;
             Pen       : (up,down);
             end;

procedure InitTurtle;
  begin
    with  Turtle do
      begin
        xPosition := 0;
        yPosition := 0;
        Richtung  := 0;
        Pen       := up;
      end;
  end;

procedure MoveTo (x,y : integer);
  begin
    with turtle do
      begin
        if Pen = down then line(xPosition,yPosition,x,y);
        xPosition := x;
        yPosition := y;
      end;
  end;


procedure PenUp;
  begin
    Turtle.Pen := up;
  end;

procedure PenDown;
  begin
    Turtle.Pen := down;
  end;


procedure HardCopy1;
  var i,j : integer;
  begin
    for i := 0 to 64 do
      begin
        write(LST,#27'A'#8);
        write(LST,#27'U'#1);
        write(LST,#27'P'#1);
        write(LST,#27'L',chr((xmax+1) mod 256),chr((xmax+1) div 256));
        for j := 0 to xmax do
          write(LST,chr(GBS[i,j]));
        writeln(LST);
      end;
  end;


procedure hardcopy;
 var answer:char;
 begin
  write(#7);write(#7);
  repeat
   ClrScr;
   gotoxy(3,5);
   writeln(' A U S D R U C K E N');
   gotoxy(7,8);
   writeln(' 1: HardCopy1');
   gotoxy(7,10);
   writeln(' e: Programmende');
   read(answer);
   case answer of
    '1' : hardcopy1;
    end;
  until answer = 'e';
 end;

{*********************************************************************}

const n = 4; h0 = 512;

var i,h,x,y,x0,y0 : integer;

{$A- }
procedure b(i:integer); forward;
procedure c(i:integer); forward;
procedure d(i:integer); forward;

procedure a (i:integer);
begin
  if i > 0 then
    begin a(i-1); x:=x+h; y:=y-h; MoveTo(x,y);
          b(i-1); x:=x+2*h; MoveTo(x,y);
          d(i-1); x:=x+h; y:= y+h; MoveTo(x,y);
          a(i-1);
    end;
end;

procedure b;
begin
  if i > 0 then
    begin b(i-1); x:=x-h; y:=y-h; MoveTo(x,y);
          c(i-1); y:=y-2*h; MoveTo(x,y);
          a(i-1); x:=x+h; y:=y-h; MoveTo(x,y);
          b(i-1);
    end;
end;

procedure  c;
begin
  if i >0 then
    begin c(i-1); x:=x-h; y:=y+h; MoveTo(x,y);
          d(i-1); x:=x-2*h; MoveTo(x,y);
          b(i-1); x:=x-h; y:=y-h; MoveTo(x,y);
          c(i-1);
    end;
end;

procedure d;
begin
  if i > 0 then
    begin d(i-1); x:= x+h; y:=y+h; MoveTo(x,y);
          a(i-1); y:=y+2*h; MoveTo(x,y);
          c(i-1); x:=x-h; y:=y+h; MoveTo(x,y);
          d(i-1);
    end;
end;

begin
  ClrGBS;
  Quad(0,0,xmax,ymax);
  i := 0;
  h := h0 div 4 ;
  x0 :=  2*h; y0:= 3*h;
  repeat
    i:=i+1;
    x0:=x0-h; h:= h div 2; y0:=y0+h;
    x:=x0; y:=y0;
    PenUp; MoveTo(x,y); PenDown;
    a(i); x:=x+h; y:=y-h; MoveTo(x,y);
    b(i); x:=x-h; y:=y-h; MoveTo(x,y);
    c(i); x:=x-h; y:=y+h; MoveTo(x,y);
    d(i); x:=x+h; y:=y+h; MoveTo(x,y);
  until i = n;
  HardCopy;
end.

