program eXtended_DIRectory;

{---------------------------------------------------------------------------
                           X D I R . P A S

  erstellt ein Gesamt-Inhaltsverzeichnis einer Diskette mit folgenden Angaben
  - Ausgabe eines Disketten-Identifikators (sofern moeglich)
  - Sortiert nach File-Namen
  - Groesse der Files in Records und kBytes
  - Anzeige von geloeschten, aber im Dir noch nicht ueberschriebenen Files
  - Anzeige von 'system'-Files
  - Ausgabe der noch verbleibenden Diskettenkapazitaet

                           Wolfgang Tesch
                              02-01-86

-----------------------------------------------------------------------------}


const
  MaxDirRecs = 17; { Groesse des Dir in Records +1 Record;
                     abhaengig vom Laufwerkstyp:
                     bei Typ 03: 16 + 1 Records ,
                     bei Typ 07: 32 + 1 Records    }
  MaxDirEintr = 68;{ 4* MaxDirRecs, da 4 Eintraege pro Sektor }

type
  Sektor     = array[0..127] of byte;
  DirSektors = array[1..MaxDirRecs] of Sektor;
  Ident      = string[10];

var
  Directory    : DirSektors; { Speicher fuer Directory }
  DMA          : Sektor;
  LwNr         : byte;
  DirEintr,
  DirSpur,
  DirSektor,
  DirRecs,
  BlkSize,
  Recs_Blk,
  Kapazitaet   : integer;

                   { =================================== }

procedure SelLaufw ( var LwNr : byte );

{ in welchem Laufwerk ist die Diskette, auf die geschrieben werden soll ? }

var Lw :string[2];

  begin
    if ParamCount = 0
      then { nehme aktuelles Laufwerk }
        LwNr := mem[$0004]
      else { untersuche 'command line parameter' }
      begin
        Lw := ParamStr(1);
        LwNr := ord(Lw[1]) - ord('A')
      end;

    { Laufwerk anmelden :}
    BDOS(14,LwNr);

    { wurde ein nicht verfuegbares Laufwerk angegeben, so wird das Programm
      an dieser Stelle mit der Meldung 'Bdos Err on ..: Select' abgebrochen }
  end;

                   { =================================== }

procedure LWParameter (var DirEintr, DirSpur, DirSector, DirRecs,
                           BlkSize,Recs_Blk,Kapazitaet : integer );

{ Berechne die Laufwerks-Charakteristika aus den Angaben im Disk-Param.-Block }

var
  DPB,
  DPB_DSM,
  DPB_BLM,
  DirSize  : integer;

begin

  DPB := BDOSHL(31);
  { Hole DPB-Adresse }

  DirEintr := 256*mem[DPB+8] + mem[DPB+7] + 1;
  { max. Anzahl der Dir-Eintraege }
  DirSpur:= 256*mem[DPB+14] + mem[DPB+13];
  { Anzahl der Spuren vor dem Dir }
   DirSektor := 1;
  { Spur-/Sektor-Nr bei denen das Dir anfaengt }
  DirRecs := DirEintr div 4;
  { Dir-Groesse in Records }

  DPB_BLM := mem[DPB+3] + 1;
  Recs_Blk := DPB_BLM;
  { Anzahl der Recs/Block }
  BlkSize := DPB_BLM shr 3;
  { Blockgroesse in kB }

  DirSize := DirRecs div DPB_BLM * BlkSize;

  DPB_DSM := 256*mem[DPB+6] + mem[DPB+5] + 1;
  Kapazitaet := DPB_DSM*BlkSize - DirSize;
  {Netto-Kapazitaet der Diskette }

end;


                   { =================================== }

procedure LiesSektor ( SpurNr,SektorNr : byte);

{ die Prozedur liest den Inhalt des durch LWNr, Spur und Sektor spezifizierten
  logischen Sektors in den vorher festgelegten Puffer }

var
  A : byte;
begin
  { Spur/Sektor auswaehlen }
  BIOS( 9,SpurNr);
  BIOS(10,SektorNr);
  { Lesen }
  A := BIOS(12);
end;



                   { =================================== }

procedure ReadDirectory (DirRecs, DirSpur, DirSektor : integer;
                         var Directory : DirSektors );

{ Lies die Directory-Information von der Diskette in den Speicher }

var i : integer;

begin
  for i := 1 to DirRecs do
  begin
    BDOS(26,addr(Directory[i]));
    { setze neuen Abschnitt von Directory als Puffer }
    LiesSektor (DirSpur, DirSektor);

    DirSektor := DirSektor mod 26;
    { 1 log. Spur = 26 log Sektoren }
    if DirSektor = 0
      then
      begin
        DirSpur := succ (DirSpur);
        DirSektor := 1;
      end
      else DirSektor := succ (DirSektor);
  end;
end;


                   { =================================== }

procedure DirVerarbeiten ( var Directory;
                          LwNr:byte;
                          DirEintr,BlkSize,Recs_Blk,Kapazitaet : integer);

{ verarbeite die im Speicher befindlichen Directory-Daten }

type
  DirEntry   = record
                 User     : byte;
                 FileName : array[1..11] of char;
                 Extent   : byte;
                 S1S2     : integer;
                 Recs     : byte;
                 BNumbers : array[0..15] of byte;
               end;

  DirBereich = array[1..MaxDirEintr] of DirEntry;

var
  Direc      : DirBereich absolute Directory;
               { ueberziehe den Speicherbereich der Directory-
                 Daten mit neuer Struktur                      }
  i,j        : integer;
  Id         : Ident;
  Files      : Integer;
  Frei,Del   : integer;
  Hilf       : DirEntry;
  LineC      : byte;


                   { ----------------------------------- }

procedure ReadIdent ( LwNr : byte ; var Id : Ident );

{ Lies Disketten-Identifikator, sofern vorhanden }

const
  T3Spur   =  98;
  T3Sektor =   1;
  T7Spur   = 195;
  T7Sektor = 13;
  CFGBYT   = $FFE9;

var
  Dma    : Sektor;
  i      : byte;

begin
  BDOS (26,addr(Dma));
  Id := '';
  if mem[CFGBYT] in [ $03,$07 ]
    then
    begin
      if mem[CFGBYT] = $03
        then LiesSektor (T3Spur,T3Sektor)
        else LiesSektor (T7Spur,T7Sektor);
      if Dma[1] <> $E5
        then for i := 0 to 9 do Id := Id + chr(Dma[i]);
    end;
end;

                   { ----------------------------------- }

procedure SortDir(var Direc:DirBereich ; DirEintr:integer);

{ sortiert den Directory-Bereich nach den Filenamen }

var
  Index,i : integer;
  Hilf    : DirEntry;

    begin
      for i := DirEintr downto 2 do
        for Index := 2 to i do
          begin
          if Direc[Index-1].FileName > Direc[Index].FileName then
            begin
              Hilf := Direc[Index-1];
              Direc[Index-1] := Direc[Index];
              Direc[Index] := Hilf;
            end;
          end;
    end;

                   { ----------------------------------- }

procedure WriteEintrag (var Entry : DirEntry;
                        var Kapazitaet , Files, Del : integer;
                        BlkSize,Recs_Blk : integer);

{ schreibt einen Eintrag des Directory }

const
  RevOn  = ^F#16;
  RevOff = ^F#04;
  DimOn  = ^F#00;
  DimOff = ^F#04;
var
  Records, Size : integer;
  Name : string[12];
  DelFlag : boolean;

begin
  write('  ');
  if Entry.Filename[10] >chr(127)
    then write(DimOn);
    { stelle 'system'-Files in 'low video' dar }
  DelFlag := false;
  if Entry.User = $E5
    then
    begin  { stelle geloeschte Files invers dar }
      write(RevOn);
      Del := succ(Del);
      DelFlag := true;
    end;
  Name := Entry.FileName;
  Insert('.',Name,9);
  write(Name);
  Records := Entry.Recs + 128*Entry.Extent;
  Size := Records div Recs_Blk;
  if Records mod Recs_Blk <> 0
    then Size := succ(Size);
  Size := Size*BlkSize;
  write('  ',Records:4,' ',Size:3);
  Files := succ(Files);
  if DelFlag then
      write(RevOff)
    else
      Kapazitaet := Kapazitaet - Size;
  write(DimOff);
end;

                   { ----------------------------------- }

begin { DirVerarbeiten }
  Direc[MaxDirEintr-3].FileName[1] := chr($E5);
  { setze Endekriterium }
  Files := 0; Del := 0;
  ReadIdent (LwNr,Id);
  ClrScr;
  writeln('Diskette ',Id,' in Laufwerk ',chr(LwNr + ord('A')),':');
  for i := 1 to 80 do write(chr(128));
  LineC := 3;

  sortDir( Direc,DirEintr);

  i:=1;
  while Direc[i].FileName[1] <> chr($E5) do
  begin { gehe Directory - Bereich durch : }
    while Direc[i].FileName = Direc[i+1].Filename do
    begin { verarbeite grosse Files mit mehreren Eintraegen }
      if Direc[i].Extent > Direc[i+1].Extent
        then
        begin
          Hilf := Direc[i];
          Direc[i] := Direc[i+1];
          Direc[i+1] := Hilf;
        end;
      i := succ(i);
    end;
    WriteEintrag(Direc[i],Kapazitaet,Files,Del,BlkSize,Recs_Blk);
    if Files mod 3 = 0
      then
      begin
        writeln;
        if LineC >23   { verhindere 'Durchlaufen' bei langer Ausgabe }
          then
          begin
            write('                         Weiter mit < RET >');
            readln;
            gotoxy(1,23);
            LineC := 1;
          end
          else LineC := succ(LineC);
      end
      else write('   ');
    i := succ(i);
  end;

  Frei := DirEintr-i+Del+1;
  if Files mod 3 <> 0 then writeln;
  for i := 1 to 80 do write(chr(128));
  write('Files:',(Files-Del):3,' +',Del:2,'   Eintraege frei:',Frei:3);
  write('    Speicherplatz frei:',Kapazitaet:3,' kBytes');
end;

                   { =================================== }

BEGIN { Hauptprog.}
  SelLaufw (LwNr);
  LWParameter(DirEintr,DirSpur,DirSektor,DirRecs,BlkSize,Recs_Blk,Kapazitaet);
  ReadDirectory(DirRecs,DirSpur,DirSektor,Directory);
  DirVerarbeiten(Directory,LwNr,DirEintr,BlkSize,Recs_Blk,Kapazitaet);
END.